/*
 * Copyright (c) 2003, B. Augestad (boa@metasystems.no)
 * COPYING RESTRICTIONS APPLY, see COPYING file.
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "clc_assert.h"

const char clc_assert_format[] = 
	"%s() (in %s, line %lu) failed assertion: %s\n";

const char clc_assert_not_null_format[] =
	"%s() (in %s, line %lu) was called with NULL for argument %s.\n";

const char clc_assert_arg_format[] =
	"%s() (in %s, line %lu) was called with "
	"an illegal argument, expected %s\n";


static int halt_policy = CLC_ASSERT_HALT_ABORT;
static int output_policy = CLC_ASSERT_OUTPUT_STDERR;

static const char* output_file = NULL;

void clc_assert_set_halt_policy(int clc_policy)
{
	halt_policy = clc_policy;
}

void clc_assert_set_output_policy(int clc_policy, const char* clc_file)
{
	output_policy = clc_policy;
	output_file = clc_file;
}

/*
 * We assume that there is enough resources to do this.
 * That may be incorrect...
 * Q: We may want to print date/time as well?
 * Q: va_lists can be used multiple times, right?
 */
void clc_assert_print(const char* clc_format, ...)
{
	int saved_errno = errno;
	va_list ap;

	if(output_policy & CLC_ASSERT_OUTPUT_STDERR) {
		va_start(ap, clc_format);
		vfprintf(stderr, clc_format, ap);
		va_end(ap);
	}

	if(output_policy & CLC_ASSERT_OUTPUT_STDOUT) {
		va_start(ap, clc_format);
		vfprintf(stdout, clc_format, ap);
		va_end(ap);
	}

	if(output_policy & CLC_ASSERT_OUTPUT_FILE) {
		if(output_file != NULL) {
			FILE* f;

			if( (f = fopen(output_file, "a")) != NULL) {
				va_start(ap, clc_format);
				vfprintf(f, clc_format, ap);
				va_end(ap);
				fclose(f);
			}
		}
	}


	errno = saved_errno;
}

void clc_assert_halt(void)
{
	switch(halt_policy) {
		case CLC_ASSERT_HALT_NO:
			/* Don't abort */
			break;

		case CLC_ASSERT_HALT_ABORT:
			abort();

		case CLC_ASSERT_HALT_EXIT:
			exit(EXIT_FAILURE);

		default:
			/*
			 * The uses has given us some illegal value.
			 * Maybe we want do abort anyway? 
			 */
			break;
	}
}

#ifdef CLC_TEST

/* Test the code above in different ways */
int main(void)
{
	char *s = NULL;
	clc_assert_set_output_policy(CLC_ASSERT_OUTPUT_STDOUT, NULL);
	clc_assert_set_halt_policy(CLC_ASSERT_HALT_NO);

	clc_assert(main, 0);
	clc_assert_arg(main, 0 == 1);
	clc_assert_not_null(main, s);

	clc_assert_set_output_policy(CLC_ASSERT_OUTPUT_NOTHING, NULL);
	clc_assert_not_null(main, s);

	clc_assert_set_output_policy(CLC_ASSERT_OUTPUT_STDERR, NULL);
	clc_assert_not_null(main, s);

	clc_assert_set_output_policy(CLC_ASSERT_OUTPUT_FILE, "/tmp/foo");
	clc_assert_not_null(main, s);

	clc_assert_set_halt_policy(CLC_ASSERT_HALT_ABORT);
	clc_assert_not_null(main, s);

	return 0;
}
	
#endif	/* CLC_TEST */
